import { isNumber, get } from 'lodash-es';
import ExcelJS from 'exceljs';

var DataFormatTypeEnum = /* @__PURE__ */ ((DataFormatTypeEnum2) => {
  DataFormatTypeEnum2["BILLION"] = "BILLION";
  DataFormatTypeEnum2["TEN_THOUSAND"] = "TEN_THOUSAND";
  DataFormatTypeEnum2["NUMBER"] = "NUMBER";
  DataFormatTypeEnum2["PERCENT"] = "PERCENT";
  DataFormatTypeEnum2["YUAN"] = "YUAN";
  return DataFormatTypeEnum2;
})(DataFormatTypeEnum || {});
const dataFormatTypeList = [
  "BILLION" /* BILLION */,
  "NUMBER" /* NUMBER */,
  "PERCENT" /* PERCENT */,
  "TEN_THOUSAND" /* TEN_THOUSAND */,
  "YUAN" /* YUAN */
];

function helperCreateToNumber(handle) {
  return function(str) {
    if (str) {
      const num = handle(str && str.replace ? str.replace(/,/g, "") : str);
      if (!isNaN(num)) {
        return num;
      }
    }
    return 0;
  };
}
const toNumber = helperCreateToNumber(parseFloat);
function commafy(num, options) {
  const {
    spaceNumber = 3,
    separator = ",",
    digits = null,
    round = true,
    ceil = false,
    floor = false
  } = options || {};
  let numStr = num.toString();
  if (typeof num === "number" && digits !== null) {
    if (round) {
      numStr = Number(num).toFixed(digits);
    } else if (ceil) {
      numStr = Math.ceil(num * Math.pow(10, digits)) / Math.pow(10, digits);
    } else if (floor) {
      numStr = Math.floor(num * Math.pow(10, digits)) / Math.pow(10, digits);
    }
  }
  const parts = numStr.toString().split(".");
  parts[0] = parts[0].replace(new RegExp(`(?=(?!(\\b))(.{${spaceNumber}})+$)`, "g"), separator);
  return parts.join(".");
}
const THOUSAND_SEPARATOR = ",";
function toFixedString(value, digits, round = true) {
  const options = {
    digits,
    round
  };
  const stringValue = commafy(value, options);
  return stringValue.replaceAll(",", "");
}
function toFixedNumber(value, digits, round = true) {
  const stringValue = toFixedString(value, digits, round);
  return toNumber(stringValue);
}
function converToTenThousandYuan(value, digits, round = true) {
  return toFixedNumber(value / 1e4, digits, round);
}
function convertToBillionYuan(value, digits, round = true) {
  return toFixedNumber(value / 1e8, digits, round);
}
function convertToPercent(value, digits, round = true) {
  return toFixedNumber(value / 0.01, digits, round);
}
function formatToCurrencyString(value, options) {
  if (isNaN(value)) {
    return options.defaultString || "";
  }
  const numberValue = value;
  const commafyOptions = {
    separator: THOUSAND_SEPARATOR,
    digits: options.digits,
    round: options.round
  };
  let resultString = commafy(numberValue, commafyOptions);
  if (!options.thousandth) {
    resultString = resultString.replaceAll(THOUSAND_SEPARATOR, "");
  }
  return resultString;
}
function getIndexConvertValueByUnitType(originalValue, convertUnitType = DataFormatTypeEnum.BILLION, digits = 2, round = true) {
  let convertValue = originalValue;
  if (!isNumber(originalValue)) {
    return convertValue;
  }
  switch (convertUnitType) {
    case DataFormatTypeEnum.TEN_THOUSAND:
      convertValue = converToTenThousandYuan(convertValue, digits, round);
      break;
    case DataFormatTypeEnum.BILLION:
      convertValue = convertToBillionYuan(convertValue, digits, round);
      break;
    case DataFormatTypeEnum.PERCENT:
      convertValue = convertToPercent(convertValue, digits, round);
      break;
  }
  return convertValue;
}
function getIndexFormatStringByUnitType(originalValue, convertUnitType) {
  const options = {
    thousandth: true,
    digits: 2,
    round: true,
    defaultString: "--"
  };
  if (isNaN(originalValue) || originalValue === null) {
    return options.defaultString || "";
  }
  if (convertUnitType === DataFormatTypeEnum.NUMBER) {
    options.digits = 0;
  }
  const convertValue = getIndexConvertValueByUnitType(originalValue, convertUnitType, options.digits);
  const formatString = formatToCurrencyString(convertValue, options);
  if (convertUnitType === DataFormatTypeEnum.PERCENT) {
    return `${formatString}%`;
  }
  return formatString;
}
function deepCopy(sourceData) {
  if (sourceData === null || typeof sourceData === "undefined" || typeof sourceData !== "object") {
    return sourceData;
  }
  if (Array.isArray(sourceData)) {
    const newArray = [];
    sourceData.forEach((item) => {
      if (typeof item === "object") {
        newArray.push(deepCopy(item));
      } else {
        newArray.push(item);
      }
    });
    return newArray;
  }
  const newObj = {};
  Object.keys(sourceData).forEach((key) => {
    if (typeof sourceData[key] === "object") {
      newObj[key] = deepCopy(sourceData[key]);
    } else {
      newObj[key] = sourceData[key];
    }
  });
  return newObj;
}

function sameType(value, type) {
  return Object.prototype.toString.call(value) === `[object ${type}]`;
}

function getDefaultConfig$1(config) {
  return Object.assign(
    {
      align: "center",
      width: 20,
      size: 10,
      bold: true,
      color: "FFFFFFFF",
      fllType: "pattern",
      fillPattern: "solid",
      fillColor: "808080",
      showBorder: true,
      borderType: "thin",
      borderColor: "FFFFFFFF"
    },
    config
  );
}
function getDefaultHeaderConfig(column) {
  return Object.assign(
    {
      headerAlign: "center",
      ...getDefaultConfig$1()
    },
    column
  );
}
function getDefaultBodyConfig(body) {
  return Object.assign(
    {
      headerRowHeight: 30,
      bodyRowHeight: 20,
      ...getDefaultConfig$1({
        bold: false,
        showBorder: false,
        color: "FF000000",
        fillPattern: "none"
      })
    },
    body
  );
}
function getDepth(node, currentDepth = 0) {
  if (!node.children || node.children.length === 0) {
    return currentDepth + 1;
  }
  return Math.max(...node.children.map((child) => getDepth(child, currentDepth + 1)));
}
function getMaxWidth(node) {
  if (!node.children || node.children.length === 0) {
    return 1;
  }
  return node.children.reduce((total, child) => total + getMaxWidth(child), 0);
}
function getMatrix(root) {
  const maxDepth = getDepth(root);
  const maxWidth = getMaxWidth(root);
  const matrix = Array.from({ length: maxDepth }, () => Array(maxWidth).fill("X"));
  function processNode(node, level, position) {
    if (level >= maxDepth || position >= maxWidth)
      return;
    matrix[level][position] = node;
    if (node.children && node.children.length) {
      let offset = position;
      node.children.forEach((child) => {
        processNode(child, level + 1, offset);
        offset += getMaxWidth(child);
      });
    }
  }
  processNode(root, 0, 0);
  return matrix;
}
function setCellStyles(cell, styles) {
  const assignStyles = {
    font: {
      bold: styles.bold,
      size: styles.size,
      color: { argb: styles.color }
    },
    fill: {
      type: styles.fllType,
      pattern: styles.fillPattern,
      fgColor: { argb: styles.fillColor }
    },
    alignment: {
      vertical: "middle",
      horizontal: styles.align
    }
  };
  if (styles.showBorder) {
    Object.assign(assignStyles, {
      border: {
        top: {
          style: styles.borderType,
          color: { argb: styles.borderColor }
        },
        // 黑色边框
        left: {
          style: styles.borderType,
          color: { argb: styles.borderColor }
        },
        bottom: {
          style: styles.borderType,
          color: { argb: styles.borderColor }
        },
        right: {
          style: styles.borderType,
          color: { argb: styles.borderColor }
        }
      }
    });
  }
  Object.assign(cell, assignStyles);
}
function buildTableHeader(context) {
  const { sheet, columns = [], body } = context;
  const matrix = getMatrix({ children: columns });
  matrix.shift();
  const headerRowsCount = matrix.length;
  const headerRows = [];
  const defaultBodyConfig = getDefaultBodyConfig(body);
  const leafNodeList = [];
  for (let i = 0; i < headerRowsCount; i++) {
    const rowItem = sheet.addRow([]);
    rowItem.height = defaultBodyConfig.headerRowHeight;
    headerRows.push(rowItem);
  }
  for (let col = 0; col < matrix[0].length; col++) {
    let row = matrix.length - 1;
    let cur = matrix[row][col];
    while (cur === "X" && row >= 0) {
      row--;
      cur = matrix[row][col];
    }
    leafNodeList.push(cur);
  }
  matrix.forEach((item, index) => {
    item.forEach((e, v) => {
      if (e === "X")
        return;
      const rowItem = headerRows[index];
      const colNumber = v + 1;
      const cell = rowItem.getCell(colNumber);
      const defaultHeaderConfig = getDefaultHeaderConfig(e);
      sheet.getColumn(colNumber).width = defaultHeaderConfig.width;
      cell.value = e.title;
      setCellStyles(cell, defaultHeaderConfig);
      cell.alignment.horizontal = defaultHeaderConfig.headerAlign;
      if (!cell.value) {
        Object.assign(cell.border, {
          diagonal: { down: true, style: "thin", color: { argb: "FF000000" } }
        });
      }
    });
  });
  for (let col = 0; col < matrix[0].length; col++) {
    for (let row = matrix.length - 1; row >= 0; row--) {
      if (matrix[row][col] === "X") {
        let cur = row - 1;
        while (cur >= 0) {
          if (matrix[cur][col] !== "X") {
            const mergeStartIndex = headerRows[cur].number;
            const mergeEndIndex = headerRows[row].number;
            const mergeColIndex = col + 1;
            sheet.mergeCells(mergeStartIndex, mergeColIndex, mergeEndIndex, mergeColIndex);
            for (let start = row; start > cur; start--) {
              matrix[start][col] = "covered";
            }
            row = cur;
            break;
          }
          cur--;
        }
      }
    }
  }
  for (let row = 0; row < headerRowsCount; row++) {
    for (let col = matrix[0].length - 1; col >= 0; col--) {
      if (matrix[row][col] === "X") {
        let cur = col - 1;
        while (cur >= 0) {
          if (matrix[row][cur] !== "X") {
            const mergeStartIndex = cur + 1;
            const mergeEndIndex = col + 1;
            const mergeRowIndex = headerRows[row].number;
            sheet.mergeCells(mergeRowIndex, mergeStartIndex, mergeRowIndex, mergeEndIndex);
            col = cur;
            break;
          }
          cur--;
        }
      }
    }
  }
  return { leafNodeList, headerRowsCount };
}
function fmtCellValue(context, value) {
  const { formatType, formatter } = context;
  if (formatter) {
    value = formatter(value);
  } else if (formatType && dataFormatTypeList.includes(formatType)) {
    value = getIndexFormatStringByUnitType(value, formatType);
  }
  return value;
}
function buildTableBody(context, leafNodeList) {
  let { sheet, body, data = [], cells = [] } = context;
  const rowItems = [];
  const render_cell_merge = [];
  data.forEach((item, rowIndex) => {
    const row = [];
    leafNodeList.forEach((col) => {
      row.push("--");
    });
    const tableItem = sheet.addRow(row);
    rowItems.push(tableItem);
    const defaultBodyConfig = getDefaultBodyConfig(body);
    tableItem.height = defaultBodyConfig.bodyRowHeight;
    tableItem.eachCell({ includeEmpty: true }, (cell, number) => {
      const colIndex = number - 1;
      const colItem = leafNodeList[colIndex];
      const { field, render } = colItem;
      const defaultHeaderConfig = getDefaultHeaderConfig(colItem);
      let cellValue = null;
      if (field)
        cellValue = get(item, field, "");
      if (render) {
        const renderResult = render({
          key: field,
          value: cellValue,
          colIndex,
          rowIndex,
          row: item
        });
        cellValue = getRenderValue(renderResult) || "";
        const merge = getMerge({
          renderResult,
          colIndex,
          rowIndex,
          headerLevel: 0
        });
        if (merge) {
          render_cell_merge.push({
            rowIndex: merge.s.r,
            colIndex: merge.s.c,
            isMerge: true,
            rowEndIndex: merge.e.r,
            colEndIndex: merge.e.c
          });
        }
      }
      cellValue = fmtCellValue(colItem, cellValue);
      cell.value = cellValue;
      setCellStyles(cell, defaultBodyConfig);
      cell.alignment.horizontal = defaultHeaderConfig.align || defaultBodyConfig.align;
    });
  });
  cells = Array.isArray(cells) ? cells : [cells];
  if (rowItems.length) {
    [...cells, ...render_cell_merge].forEach((item) => {
      let { rowIndex, colIndex, field, value, isMerge, rowEndIndex, colEndIndex } = item;
      let cellValue;
      const defaultCellConfig = getDefaultBodyConfig(item);
      if (rowIndex !== void 0 && rowIndex >= 0) {
        const rowItem = rowItems[rowIndex];
        const rowNumber = rowItem.number;
        if (colIndex === void 0) {
          rowItem.eachCell({ includeEmpty: true }, (cell, colNumber) => {
            const columnItem = leafNodeList[colNumber - 1];
            const key = field || columnItem.field;
            key && (cellValue = data[rowIndex][key]);
            cell.value = fmtCellValue(item, cellValue);
            setCellStyles(cell, defaultCellConfig);
          });
          isMerge && sheet.mergeCells(rowNumber, 1, rowNumber, leafNodeList.length);
        } else if (colIndex !== void 0 && colIndex >= 0) {
          if (rowEndIndex !== void 0 && rowEndIndex >= rowIndex && colEndIndex === void 0) {
            const rows = rowItems.slice(rowIndex, rowEndIndex + 1);
            const rowEndNumber = rowItems[rowEndIndex].number;
            const colNumber = colIndex + 1;
            const columnItem = leafNodeList[colIndex];
            rows.forEach((rowItem2, index) => {
              const cell = rowItem2.getCell(colNumber);
              const key = field || columnItem.field;
              key && (cellValue = data[rowIndex + index][key]);
              cell.value = fmtCellValue(item, cellValue);
              setCellStyles(cell, defaultCellConfig);
            });
            isMerge && sheet.mergeCells(rowNumber, colNumber, rowEndNumber, colNumber);
          } else if (colEndIndex !== void 0 && colEndIndex >= colIndex && rowEndIndex === void 0) {
            rowItem.eachCell({ includeEmpty: true }, (cell, colNumber) => {
              if (colNumber - 1 < colIndex || colNumber - 1 > colEndIndex)
                return;
              const columnItem = leafNodeList[colNumber - 1];
              const key = field || columnItem.field;
              key && (cellValue = data[rowIndex][key]);
              cell.value = fmtCellValue(item, cellValue);
              setCellStyles(cell, defaultCellConfig);
            });
            isMerge && sheet.mergeCells(rowNumber, colIndex + 1, rowNumber, colEndIndex + 1);
          } else if (colEndIndex !== void 0 && colEndIndex >= colIndex && rowEndIndex !== void 0 && rowEndIndex >= rowIndex) {
            const rows = rowItems.slice(rowIndex, rowEndIndex + 1);
            const rowEndNumber = rowItems[rowEndIndex].number;
            rows.forEach((rowItem2, index) => {
              rowItem2.eachCell({ includeEmpty: true }, (cell, colNumber) => {
                if (colNumber - 1 < colIndex || colNumber - 1 > colEndIndex)
                  return;
                const columnItem = leafNodeList[colNumber - 1];
                const key = field || columnItem.field;
                key && (cellValue = data[rowIndex + index][key]);
                cell.value = fmtCellValue(item, cellValue);
                setCellStyles(cell, defaultCellConfig);
              });
            });
            isMerge && sheet.mergeCells(rowNumber, colIndex + 1, rowEndNumber, colEndIndex + 1);
          } else {
            const cell = rowItem.getCell(colIndex);
            const columnItem = leafNodeList[colIndex];
            const key = field || columnItem.field;
            key && (cellValue = data[rowIndex][key]);
            value && (cellValue = value);
            cell.value = fmtCellValue(item, cellValue);
            setCellStyles(cell, defaultCellConfig);
          }
        }
      }
    });
  }
}
const getRenderValue = (renderResult) => {
  const values = [];
  const dealChildren = (value) => {
    const child = value || "";
    if (sameType(child, "String") || sameType(child, "Number")) {
      values.push(child);
    }
    if (sameType(child, "Object")) {
      const { children: children2 } = child.props;
      if (sameType(children2, "String") || sameType(children2, "Number")) {
        values.push(children2);
      }
      if (sameType(children2, "Object")) {
        dealChildren(children2);
      }
      if (sameType(children2, "Array")) {
        children2.forEach((_child) => {
          dealChildren(_child);
        });
      }
    }
    if (sameType(child, "Array")) {
      child.forEach((_child) => {
        dealChildren(_child);
      });
    }
  };
  const children = get(renderResult, "children", null);
  if (children) {
    dealChildren(children);
  } else {
    dealChildren(renderResult);
  }
  return values.join("");
};
const getMerge = ({ renderResult, colIndex, rowIndex, headerLevel }) => {
  const props = get(renderResult, "props", false);
  if (!props)
    return false;
  const { colSpan, rowSpan } = props;
  if (colSpan && colSpan !== 1 || rowSpan && rowSpan !== 1) {
    const realRowIndex = rowIndex + headerLevel;
    return {
      s: { c: colIndex, r: realRowIndex },
      e: {
        c: colIndex + (colSpan || 1) - 1,
        r: realRowIndex + (rowSpan || 1) - 1
      }
    };
  }
};
function buildNormalBody(context) {
  const { leafNodeList, headerRowsCount } = buildTableHeader(context);
  buildTableBody(context, leafNodeList);
}

var BodyTypeEnum = /* @__PURE__ */ ((BodyTypeEnum2) => {
  BodyTypeEnum2["NORMAL"] = "normal";
  return BodyTypeEnum2;
})(BodyTypeEnum || {});
const bodyTypeMap = /* @__PURE__ */ new Map([["normal" /* NORMAL */, buildNormalBody]]);

function buildNormalFooter(context) {
}

const footerTypeMap = /* @__PURE__ */ new Map([
  ["normal" /* NORMAL */, buildNormalFooter]
]);

function getDefaultConfig(header) {
  return Object.assign(
    {
      height: 30,
      size: 16,
      bold: true,
      align: "center"
    },
    header
  );
}
function buildNormalHeader(context) {
  const { sheet, name, header, context: contextInfo } = context;
  const defaultConfig = getDefaultConfig(header);
  const mergeCols = contextInfo.columnleafNodeTotal || 1;
  const headerTitleRow = sheet.addRow([name]);
  const headerTitleRowNumber = headerTitleRow.number;
  headerTitleRow.height = defaultConfig.height;
  headerTitleRow.font = {
    size: defaultConfig.size,
    bold: defaultConfig.bold
  };
  headerTitleRow.alignment = {
    vertical: "middle",
    horizontal: defaultConfig.align
  };
  sheet.mergeCells(headerTitleRowNumber, 1, headerTitleRowNumber, mergeCols);
}

const headerTypeMap = /* @__PURE__ */ new Map([
  ["normal" /* NORMAL */, buildNormalHeader]
]);

function buildNormalSide(context) {
}

const sideTypeMap = /* @__PURE__ */ new Map([["normal" /* NORMAL */, buildNormalSide]]);

function buildNormalTable(context) {
  const { headerType, bodyType = BodyTypeEnum.NORMAL, sideType, footerType } = context;
  const headerFn = headerTypeMap.get(headerType);
  const bodyFn = bodyTypeMap.get(bodyType);
  const sideFn = sideTypeMap.get(sideType);
  const footerFn = footerTypeMap.get(footerType);
  headerFn && headerFn(context);
  bodyFn && bodyFn(context);
  sideFn && sideFn(context);
  footerFn && footerFn(context);
}

var TableTypeEnum = /* @__PURE__ */ ((TableTypeEnum2) => {
  TableTypeEnum2["NORMAL"] = "normal";
  return TableTypeEnum2;
})(TableTypeEnum || {});
const tableTypeMap = /* @__PURE__ */ new Map([
  ["normal" /* NORMAL */, buildNormalTable]
]);

function useBuildExcel(tableName, context, onlyOneSheet) {
  let workbook;
  let sheetContext;
  function getDefaultContext() {
    return {
      columnleafNodeTotal: 0
    };
  }
  function getLeafNodeTotal(data = []) {
    const queue = [...data];
    const leafNodes = [];
    while (queue.length) {
      const node = queue.shift();
      if (!node.children || node.children && !node.children.length) {
        leafNodes.push(node);
      }
      if (node.children && node.children.length) {
        queue.push(...node.children);
      }
    }
    return leafNodes.length;
  }
  async function handleItem(item) {
    const {
      tableType = TableTypeEnum.NORMAL,
      beforeExport,
      name = "normal",
      columns,
      context: context2 = getDefaultContext()
    } = item;
    const sheet = onlyOneSheet ? sheetContext : workbook.addWorksheet(name);
    context2.columnleafNodeTotal = getLeafNodeTotal(columns);
    item.excelJs = workbook;
    item.sheet = sheet;
    item.context = context2;
    const useTableFn = tableTypeMap.get(tableType);
    if (useTableFn)
      useTableFn(item);
    if (beforeExport && typeof beforeExport === "function") {
      await beforeExport(item);
    }
  }
  async function handleContext() {
    if (!Array.isArray(context)) {
      context = [context];
    }
    onlyOneSheet && (sheetContext = workbook.addWorksheet(tableName));
    for (let item of context) {
      await handleItem(item);
    }
  }
  async function exportFile() {
    const buffer = await workbook.xlsx.writeBuffer();
    const blob = new Blob([buffer], {
      type: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
    });
    const url = URL.createObjectURL(blob);
    const downloadLink = document.createElement("a");
    downloadLink.href = url;
    downloadLink.download = tableName;
    document.body.appendChild(downloadLink);
    downloadLink.click();
    downloadLink.remove();
    URL.revokeObjectURL(url);
  }
  async function buildExcel() {
    workbook = new ExcelJS.Workbook();
    await handleContext();
    await exportFile();
  }
  return {
    buildExcel
  };
}

function useExtend(map) {
  function before(type, fn) {
    const strategy = map.get(type);
    if (strategy) {
      map.set(type, (context) => {
        fn(context);
        strategy(context);
      });
    }
  }
  function after(type, fn) {
    const strategy = map.get(type);
    if (strategy) {
      map.set(type, (context) => {
        strategy(context);
        return fn(context);
      });
    }
  }
  function extend(type, fn) {
    map.set(type, fn);
  }
  return {
    before,
    after,
    extend
  };
}
const header = useExtend(headerTypeMap);
const body = useExtend(bodyTypeMap);
const footer = useExtend(footerTypeMap);
const side = useExtend(sideTypeMap);
const table = useExtend(tableTypeMap);

function pixelsHeightToPoints(pixels) {
  return pixels * 72 / 96;
}
function pointsToPixels(points) {
  return points * 96 / 72;
}
function pixelsToWidth(pixels) {
  return pixels / 11;
}
const pixelUtil = {
  pixelsHeightToPoints,
  pointsToPixels,
  pixelsToWidth
};

async function parseFile({ file }) {
  if (!file)
    return Promise.reject("no file");
  async function readExcelFile(file2) {
    return new Promise((resolve, reject) => {
      const reader = new FileReader();
      reader.onload = function(event) {
        resolve(event.target.result);
      };
      reader.onerror = function(error) {
        reject(error);
      };
      reader.readAsArrayBuffer(file2);
    });
  }
  try {
    const data = await readExcelFile(file);
    const workbook = new ExcelJS.Workbook();
    await workbook.xlsx.load(data);
    const table = {};
    workbook.eachSheet(function(worksheet, sheetId) {
      let firstRow = worksheet.getRow(1);
      if (!firstRow.cellCount)
        return;
      let keys = (firstRow.values || []).filter(Boolean);
      const sheet_data = [];
      worksheet.eachRow(function(row, rowNumber) {
        if (rowNumber == 1)
          return;
        let values = row.values;
        let obj = {};
        for (let i = 1; i < keys.length; i++) {
          obj[keys[i]] = values[i];
        }
        sheet_data.push(obj);
      });
      const sheetName = worksheet.name;
      table[sheetId + "_" + sheetName] = {
        sheetId,
        sheetName,
        columns: keys,
        data: sheet_data
      };
    });
    return {
      table,
      workbook
    };
  } catch (error) {
    return Promise.reject(error.message || "excel 解析失败");
  }
}

async function exportFile({ fileName, columns, dataSource, sheetNames, ...other }) {
  function fmtCol(columns2) {
    return columns2.map((v) => {
      const { key, dataIndex, field, children, width, ...other2 } = v;
      const res = {
        ...other2,
        field: key || dataIndex || field
      };
      if (width) {
        res["width"] = pixelUtil.pixelsToWidth(width);
      }
      if (children) {
        res.children = fmtCol(children);
      }
      return res;
    });
  }
  const name = fileName.replace(".xlsx", "");
  let opt = [];
  if (sheetNames && typeof sheetNames !== "string") {
    sheetNames.forEach((sheet, index) => {
      const _column = columns[index];
      opt.push({
        name: sheet,
        columns: fmtCol(_column),
        data: dataSource[index],
        ...other
      });
    });
  } else {
    const _columns = fmtCol(columns);
    const sheetName = sheetNames || "sheet 1";
    opt.push({
      name: sheetName,
      columns: _columns,
      data: dataSource,
      ...other
    });
  }
  return exportExcel(name, opt);
}

async function exportExcel(tableName, options, onlyOneSheet) {
  options = deepCopy(options);
  const { buildExcel } = useBuildExcel(tableName, options, onlyOneSheet);
  await buildExcel();
}

export { body, exportExcel, exportFile, footer, header, parseFile, pixelUtil, side, table };
